@@ -4,17 +4,9 @@
 #  --> https://CRAN.R-project.org/package=spam              #
 #  --> https://CRAN.R-project.org/package=spam64            #
 #  --> https://git.math.uzh.ch/reinhard.furrer/spam         #
-# by Reinhard Furrer [aut, cre], Florian Gerber [ctb],      #
-#    Daniel Gerber [ctb], Kaspar Moesinger [ctb],           #
-#    Youcef Saad [ctb] (SPARSEKIT),                         #
-#    Esmond G. Ng [ctb] (Fortran Cholesky routines),        #
-#    Barry W. Peyton [ctb] (Fortran Cholesky routines),     #
-#    Joseph W.H. Liu [ctb] (Fortran Cholesky routines),     #
-#    Alan D. George [ctb] (Fortran Cholesky routines),      #
-#    Esmond G. Ng [ctb] (Fortran Cholesky routines),        #
-#    Barry W. Peyton [ctb] (Fortran Cholesky routines),     #
-#    Joseph W.H. Liu [ctb] (Fortran Cholesky routines),     #
-#    Alan D. George [ctb] (Fortran Cholesky routines)       #
+# by Reinhard Furrer [aut, cre], Florian Gerber [aut],      #
+#    Roman Flury [aut], Daniel Gerber [ctb],                #
+#    Kaspar Moesinger [ctb]                                 #
 # HEADER END ################################################
 
 
@@ -22,14 +14,14 @@
 
 "dim<-.spam" <- function(x, value) {
     if (is.spam(x)) {
-        
+
         dimx <- x@dimension
         pdim <- prod(dimx)
         vlen <- prod(value)
         if( !identical(pdim,vlen))
             stop( sprintf("dims [product %d] do not match the length of object [%d]. Do you want `pad`",
                           pdim,vlen))
-        
+
         if (length(value)>2)
             stop("dims should be of length 1 or 2")
         if (identical(length(value),1L))
@@ -37,18 +29,18 @@
 
         if(any(dimx<1))
             stop("the dims contain negative values")
-        
+
         tmp <- cbind(st=rep(1:dim(x)[1],diff(x@rowpointers)), nd=x@colindices)
         ind <- tmp[,1]+(tmp[,2]-1)*dimx[1] - 1
 
         slist <- list(i = ind%%value[1]   +1,
                        j = ind%/%value[1] +1,
                        x@entries)
-        
+
         return( spam.list( slist, nrow=value[1], ncol=value[2],
                           eps = .Machine$double.eps))
-        
-        
+
+
     } else  {
         dim(x) <- value
         x
@@ -68,10 +60,10 @@
         stop("dims should be postive integers.")
     if (!identical( length(value), 2L))
         stop("dims should be of length 2.")
-    
+
     dimx <- x@dimension
     last <- value[1]+1
-    
+
   # In three steps:
   #  1) Address col truncation
             # to safe time, we also take into account if we have fewer or equal rows
@@ -79,7 +71,7 @@
   #  3) if fewer rows and more columns, truncate
   # In any case, dimensions are fixed at the end.
 
-  
+
   # If fewer cols required, we run reducedim
   if (dimx[2]>value[2]){
 
@@ -89,7 +81,7 @@
           SS <- .format64()
       else
           SS <- .format32
-      
+
       z <- .C64("reducedim",
                 SIGNATURE=c("double", SS$signature, SS$signature,
                     "double", SS$signature, SS$signature, SS$signature,
@@ -97,29 +89,29 @@
                 oldra = x@entries,
                 oldja = x@colindices,
                 oldia = x@rowpointers,
-                
+
                 eps = getOption("spam.eps"),
                 min(value[1],dimx[1]),
                 value[2],
                 nz = 1,
-                
+
                 entries=vector_dc("double",length(x@entries)),
                 colindices=vector_dc(SS$type,length(x@entries)),
                 rowpointers=vector_dc(SS$type,last),
 
-                INTENT=c("r", "r", "r", 
-                         "r", "r", "r", "w", 
+                INTENT=c("r", "r", "r",
+                         "r", "r", "r", "w",
                          "w", "w", "w"),
                 NAOK = getOption("spam.NAOK"),
                 PACKAGE = SS$package)
-                
+
     if (z$nz==1 ){ #was identical( z$nz,1L)
         ## print("2")
             return(
                 .newSpam(
-                    entries=x@entries,
-                    colindices=x@colindices,
-                    rowpointers=c(1,rep_len64(2,value[1])), 
+                    # entries=x@entries,
+                    # colindices=x@colindices,
+                    # rowpointers=c(1,rep_len64(2,value[1])),
                     dimension=value,
                     force64=force64
                     )
@@ -129,7 +121,7 @@
       x <- .newSpam(
           entries=z$entries[1:nz],
           colindices=z$colindices[1:nz],
-          rowpointers=z$rowpointers[1:min(last,dimx[1]+1)], 
+          rowpointers=z$rowpointers[1:min(last,dimx[1]+1)],
           dimension=value, #actually here dim 2 = value 2 but dim1 maybe not yet
           force64=force64
           )
@@ -148,7 +140,7 @@
   }
 
     # special case: fewer rows and more columns, truncate
-  if((dimx[1]>=value[1])&(dimx[2]<=value[2])) { ## added =, think about it again 
+  if((dimx[1]>=value[1])&(dimx[2]<=value[2])) { ## added =, think about it again
       ## print("4")
       lastelement <- (x@rowpointers[last]-1)
 
@@ -176,8 +168,8 @@ setMethod("pad<-",   "matrix",
           function(x, value) {
               if (!identical( length(value), 2L)) stop("dims should be of length 2.")
               tmp <- matrix(0, value)
-              mr <- 1:min(value[1], nrow(x))
-              mc <- 1:min(value[2], ncol(x))
+              mr <- 1:min(value[1], dim(x)[1])
+              mc <- 1:min(value[2], dim(x)[2])
               tmp[mr,mc] <- x[mr,mc]
               return(tmp)
           })
